/**
  ******************************************************************************
  * @file    sd_diskio.c
  * @author  MCD Application Team
  * @brief   SD Disk I/O driver.
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2017 STMicroelectronics International N.V.
  * All rights reserved.</center></h2>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted, provided that the following conditions are met:
  *
  * 1. Redistribution of source code must retain the above copyright notice,
  *    this list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  * 3. Neither the name of STMicroelectronics nor the names of other
  *    contributors to this software may be used to endorse or promote products
  *    derived from this software without specific written permission.
  * 4. This software, including modifications and/or derivative works of this
  *    software, must execute solely and exclusively on microcontroller or
  *    microprocessor devices manufactured by or for STMicroelectronics.
  * 5. Redistribution and use of this software other than as permitted under
  *    this license is void and will automatically terminate your rights under
  *    this license.
  *
  * THIS SOFTWARE IS PROVIDED BY STMICROELECTRONICS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS, IMPLIED OR STATUTORY WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  * PARTICULAR PURPOSE AND NON-INFRINGEMENT OF THIRD PARTY INTELLECTUAL PROPERTY
  * RIGHTS ARE DISCLAIMED TO THE FULLEST EXTENT PERMITTED BY LAW. IN NO EVENT
  * SHALL STMICROELECTRONICS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
  * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "ff_gen_drv.h"
#include "Driver_diskio.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
extern MSD_CARDINFO CardInfo;
#define SD_DEFAULT_BLOCK_SIZE 512

/*
 * Depending on the usecase, the SD card initialization could be done at the
 * application level, if it is the case define the flag below to disable
 * the BSP_SD_Init() call in the SD_Initialize().
 */

/* #define DISABLE_SD_INIT */

/* Private variables ---------------------------------------------------------*/
/* Disk status */
static volatile DSTATUS Stat = STA_NOINIT;

/* Private function prototypes -----------------------------------------------*/
static DSTATUS SD_CheckStatus(BYTE lun);
DSTATUS SD_initialize (BYTE);
DSTATUS SD_status (BYTE);
DRESULT SD_read (BYTE, BYTE*, DWORD, UINT);
#if _USE_WRITE == 1
  DRESULT SD_write (BYTE, const BYTE*, DWORD, UINT);
#endif /* _USE_WRITE == 1 */
#if _USE_IOCTL == 1
  DRESULT SD_ioctl (BYTE, BYTE, void*);
#endif  /* _USE_IOCTL == 1 */

const Diskio_drvTypeDef  SD_Driver =
{
  SD_initialize,
  SD_status,
  SD_read,
#if  _USE_WRITE == 1
  SD_write,
#endif /* _USE_WRITE == 1 */

#if  _USE_IOCTL == 1
  SD_ioctl,
#endif /* _USE_IOCTL == 1 */
};

/* Private functions ---------------------------------------------------------*/
static DSTATUS SD_CheckStatus(BYTE lun)
{
  return 0;
}

/**
  * @brief  Initializes a Drive
  * @param  lun : not used
  * @retval DSTATUS: Operation status
  */
DSTATUS SD_initialize(BYTE lun)
{
  Stat = STA_NOINIT;
  Stat = MSD_Init();
  if(Stat == 0)	  return 0;
  else 	  return STA_NOINIT;
}

/**
  * @brief  Gets Disk Status
  * @param  lun : not used
  * @retval DSTATUS: Operation status
  */
DSTATUS SD_status(BYTE lun)
{
  return SD_CheckStatus(lun);
}

/**
  * @brief  Reads Sector(s)
  * @param  lun : not used
  * @param  *buff: Data buffer to store read data
  * @param  sector: Sector address (LBA)
  * @param  count: Number of sectors to read (1..128)
  * @retval DRESULT: Operation result
  */
DRESULT SD_read(BYTE lun, BYTE *buff, DWORD sector, UINT count)
{
    int res;
    if( !count )
    {    
        return RES_PARERR;  /* countܵ0򷵻ز */
    }
  
    if(count==1)            /* 1sectorĶ */      
    {       
        res =  MSD_ReadSingleBlock(sector ,buff );
    }else                    /* sectorĶ */     
    {   
        res = MSD_ReadMultiBlock(sector , buff ,count);
    }
        
    if(res == 0)
    {
        return RES_OK;
    }else
    {
        return RES_ERROR;
    }

}

/**
  * @brief  Writes Sector(s)
  * @param  lun : not used
  * @param  *buff: Data to be written
  * @param  sector: Sector address (LBA)
  * @param  count: Number of sectors to write (1..128)
  * @retval DRESULT: Operation result
  */
#if _USE_WRITE == 1
DRESULT SD_write(BYTE lun, const BYTE *buff, DWORD sector, UINT count)
{
    int res;
    if( !count )
    {    
        return RES_PARERR;  /* countܵ0򷵻ز */
    }
    if(count==1)            /* 1sectorд */      
    {   
        res = MSD_WriteSingleBlock(sector , (uint8_t *)(&buff[0]) ); 
    }else                    /* sectorд */    
    {   
        res = MSD_WriteMultiBlock(sector , (uint8_t *)(&buff[0]) , count );	  
    }                                                
    if(res == 0)
    {
        return RES_OK;
    }else
    {
       return RES_ERROR;
    }
}
#endif /* _USE_WRITE == 1 */

/**
  * @brief  I/O control operation
  * @param  lun : not used
  * @param  cmd: Control code
  * @param  *buff: Buffer to send/receive control data
  * @retval DRESULT: Operation result
  */
#if _USE_IOCTL == 1
DRESULT SD_ioctl(BYTE lun, BYTE cmd, void *buff)
{
  DRESULT res = RES_ERROR;
  
  switch (cmd)
  {
  /* Make sure that no pending write process */
  case CTRL_SYNC :
    res = RES_OK;
    break;

  /* Get number of sectors on the disk (DWORD) */
  case GET_SECTOR_COUNT :
    MSD_GetCardInfo(&CardInfo);
    *(DWORD*)buff = CardInfo.Capacity/CardInfo.BlockSize;
    res = RES_OK;
    break;

  /* Get R/W sector size (WORD) */
  case GET_SECTOR_SIZE :
    MSD_GetCardInfo(&CardInfo);
    *(WORD*)buff = CardInfo.BlockSize;
    res = RES_OK;
    break;

  /* Get erase block size in unit of sector (DWORD) */
  case GET_BLOCK_SIZE :
    MSD_GetCardInfo(&CardInfo);
    *(WORD*)buff = CardInfo.BlockSize;
	res = RES_OK;
    break;

  default:
    res = RES_PARERR;
  }

  return res;
}
#endif /* _USE_IOCTL == 1 */




////=======================================================================
////=======================================================================
////Flash W25Qxx
////=======================================================================
//
///* Private function prototypes -----------------------------------------------*/
//static DSTATUS Flash_CheckStatus(BYTE lun);
//DSTATUS Flash_initialize (BYTE);
//DSTATUS Flash_status (BYTE);
//DRESULT Flash_read (BYTE, BYTE*, DWORD, UINT);
//#if _USE_WRITE == 1
//  DRESULT Flash_write (BYTE, const BYTE*, DWORD, UINT);
//#endif /* _USE_WRITE == 1 */
//#if _USE_IOCTL == 1
//  DRESULT Flash_ioctl (BYTE, BYTE, void*);
//#endif  /* _USE_IOCTL == 1 */
//
//const Diskio_drvTypeDef  Flash_Driver =
//{
//  Flash_initialize,
//  Flash_status,
//  Flash_read,
//#if  _USE_WRITE == 1
//  Flash_write,
//#endif /* _USE_WRITE == 1 */
//
//#if  _USE_IOCTL == 1
//  Flash_ioctl,
//#endif /* _USE_IOCTL == 1 */
//};
//
///* Private functions ---------------------------------------------------------*/
//static DSTATUS Flash_CheckStatus(BYTE lun)
//{
//  return 0;
//}
//
///**
//  * @brief  Initializes a Drive
//  * @param  lun : not used
//  * @retval DSTATUS: Operation status
//  */
//DSTATUS Flash_initialize(BYTE lun)
//{
//  Stat = STA_NOINIT;
//  Stat  = W25QXX_Init();
//  if(Stat == 0)	  return 0;
//  else 	  return STA_NOINIT;
//}
//
///**
//  * @brief  Gets Disk Status
//  * @param  lun : not used
//  * @retval DSTATUS: Operation status
//  */
//DSTATUS Flash_status(BYTE lun)
//{
//  return Flash_CheckStatus(lun);
//}
//
///**
//  * @brief  Reads Sector(s)
//  * @param  lun : not used
//  * @param  *buff: Data buffer to store read data
//  * @param  sector: Sector address (LBA)
//  * @param  count: Number of sectors to read (1..128)
//  * @retval DRESULT: Operation result
//  */
//DRESULT Flash_read(BYTE lun, BYTE *buff, DWORD sector, UINT count)
//{
////    DRESULT res = RES_ERROR;
//    if( !count )
//    {
//        return RES_PARERR;  /* countܵ0򷵻ز */
//    }
//
//    W25QXX_Read(sector<<9,buff,count<<9);
//    return RES_OK;
//}
//
///**
//  * @brief  Writes Sector(s)
//  * @param  lun : not used
//  * @param  *buff: Data to be written
//  * @param  sector: Sector address (LBA)
//  * @param  count: Number of sectors to write (1..128)
//  * @retval DRESULT: Operation result
//  */
//#if _USE_WRITE == 1
//DRESULT Flash_write(BYTE lun, const BYTE *buff, DWORD sector, UINT count)
//{
////    DRESULT res = RES_ERROR;
//    //W25QXX_EraseBlock(sector<<12);
//    //W25QXX_WritePage(sector<<12,(BYTE *)buff,count<<12);
//    W25QXX_Write(sector<<9,(BYTE *)buff,count<<9);
//    return RES_OK;
//
//}
//#endif /* _USE_WRITE == 1 */
//
///**
//  * @brief  I/O control operation
//  * @param  lun : not used
//  * @param  cmd: Control code
//  * @param  *buff: Buffer to send/receive control data
//  * @retval DRESULT: Operation result
//  */
//#if _USE_IOCTL == 1
//DRESULT Flash_ioctl(BYTE lun, BYTE cmd, void *buff)
//{
//  DRESULT res = RES_ERROR;
//
//  switch (cmd)
//  {
//  /* Make sure that no pending write process */
//  case CTRL_SYNC :
//    res = RES_OK;
//    break;
//
//  /* Get number of sectors on the disk (DWORD) */
//  case GET_SECTOR_COUNT :
//    *(DWORD*)buff = 512*8;    //1M=256,2M=512,4M=1024,8M=2048  *8
//    res = RES_OK;
//    break;
//
//  /* Get R/W sector size (WORD) */
//  case GET_SECTOR_SIZE :
//    *(WORD*)buff = 512;
//    res = RES_OK;
//    break;
//
//  /* Get erase block size in unit of sector (DWORD) */
//  case GET_BLOCK_SIZE :
//    *(WORD*)buff = 4096;
//	res = RES_OK;
//    break;
//
//  default:
//    res = RES_PARERR;
//  }
//
//  return res;
//}
//#endif /* _USE_IOCTL == 1 */


//=============================================================================
DRESULT FATFS_Init(void)
{
    FATFS_LinkDriver(&SD_Driver,"0:");
    //FATFS_LinkDriver(&Flash_Driver,"1:");
    return  RES_OK;
}


/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/

